﻿/* Used APIs and minimal browser version:
30 browserAction.onClicked
30 commands.onCommand
30 contextMenus.create
30 contextMenus.onClicked
30 i18n.getMessage
30 management.uninstallSelf
30 notifications.clear
30 notifications.create
30 runtime.id
30 runtime.lastError
30 runtime.onInstalled
30 runtime.sendNativeMessage
30 storage.local.get
30 tabs.create
30 tabs.remove
46 tabs.TAB_ID_NONE

Needed: 46
*/


function urlTest(url) {
    try {
        new URL(url);
        return true;
    } catch (e) {
        return false;
    }
}

function showNotification(message) {
    chrome.notifications.create('isNotUrl', {
        type: 'basic',
        iconUrl: '/icons/icon_64x64.png',
        title: chrome.i18n.getMessage('extName'),
        message: message,
    });
    setTimeout(() => chrome.notifications.clear('isNotUrl'), 3000);
}

// Opening routine with checking
function startMpv(url, senderIsTab = false) {
    if (!urlTest(url)) {
        console.info('"%s" is not valid URL!', url);
        showNotification(chrome.i18n.getMessage('isNotUrl'));
        return;
    }

    if (url.startsWith('chrome') || url.startsWith('about:') || url.startsWith('moz-extension:'))
        return;

    chrome.storage.local.get(['closeAfterMPV', 'mpvParameters'], (currentSettings) => {
        if (senderIsTab && currentSettings.closeAfterMPV)
            chrome.tabs.query({currentWindow: true, active: true}, function (tabs) {
                const currentTab = tabs[0];
                if (currentTab && currentTab.id !== chrome.tabs.TAB_ID_NONE)
                    chrome.tabs.remove(currentTab.id);
            });


        let params = `${currentSettings.mpvParameters} "${url}"`;

        console.info('Starting MPV with: "%s"', params);
        chrome.runtime.sendNativeMessage('com.ahohnmyc.link2mpv', {prms: params}, (govno) => {
            const le = chrome.runtime.lastError;
            if (le) {
                switch (le.message) {
                    case 'Specified native messaging host not found.':
                        showNotification(chrome.i18n.getMessage('nmhNotFound'));
                        chrome.tabs.create({url: './info/link2mpv.html'});
                    case 'Native host has exited.':
                        break;
                    default:
                        showNotification(le.message);
                        console.error(le.message);
                        break;
                }
            } else {
                if (govno.hasOwnProperty('error')) {
                    switch (govno.error) {
                        case 'MPV not found!':
                            showNotification(chrome.i18n.getMessage('mpvNotFound'));
                            chrome.tabs.create({url: './info/mpv.html'});
                            break;
                        default:
                            showNotification(govno.error);
                            console.error(govno.error);
                            break;
                    }
                }
            }
        });
        console.debug('------------------------------')
    });

}

function sendCurrentTabToMPV() {
    chrome.tabs.query({currentWindow: true, active: true}, (tabs) => {
        const currentTab = tabs[0];
        startMpv(currentTab.url, true);
    });
}


chrome.runtime.onInstalled.addListener(() => {
    // Init storage
    // https://stackoverflow.com/a/23638867
    chrome.storage.local.get(
        {
            closeAfterMPV: false,
            mpvParameters: '--ytdl-format="[height<=720]"'
        },
        (data) => chrome.storage.local.set(data)
    );

    chrome.tabs.create({url: './info/info.html'});
});


// Init context menus
chrome.contextMenus.create({
    id: 'tab2mpv',
    title: chrome.i18n.getMessage('playTabInMPV'),
    contexts: ['page', 'frame']
});

['selection', 'link', 'image', 'audio', 'video'].forEach((type) => {
    chrome.contextMenus.create({
        id: type + '2mpv',
        title: chrome.i18n.getMessage(`play${type[0].toUpperCase() + type.slice(1)}InMPV`),
        contexts: [type]
    });
});


// Menu handler
chrome.contextMenus.onClicked.addListener((info, tab) => {
    let senderIsTab = false;
    let url;

    switch (info.menuItemId) {
        case 'tab2mpv':
            senderIsTab = true;
            url = info.frameUrl || info.pageUrl;
            break;
        case 'link2mpv':
            url = info.linkUrl;
            break;
        case 'image2mpv':
        case 'audio2mpv':
        case 'video2mpv':
            url = info.srcUrl;
            break;
        case 'selection2mpv':
            url = info.selectionText;
            break;
        default:
            break;
    }

    console.debug('Info: %o\nTab: %o', info, tab);

    startMpv(url, senderIsTab)
});


// Icon on toolbar click
chrome.browserAction.onClicked.addListener(({'url': url}) => startMpv(url));


// Shortcut handler
chrome.commands.onCommand.addListener((command) => {
    console.info('Got command by hotkey:', command);
    switch (command) {
        case 'link2mpv':
            sendCurrentTabToMPV();
            break;
    }
});
